@@ -26,10 +26,10 @@ use Image::ExifTool qw(:Utils :Vars);
 use Image::ExifTool::Shortcuts;
 use Image::ExifTool::HTML qw(EscapeHTML);
 use Image::ExifTool::IPTC;
+use Image::ExifTool::Canon;
 use Image::ExifTool::Nikon;
-use Image::ExifTool::Real;
 
-$VERSION = '1.56';
+$VERSION = '1.65';
 @ISA = qw(Exporter);
 
 sub NumbersFirst;
@@ -46,6 +46,7 @@ my $docType = q{<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
 my $caseInsensitive;    # flag to ignore case when sorting tag names
 
 # Descriptions for the TagNames documentation
+# (descriptions may also be defined in tag table NOTES)
 # Note: POD headers in these descriptions start with '~' instead of '=' to keep
 # from confusing POD parsers which apparently parse inside quoted strings.
 my %docs = (
@@ -66,11 +67,11 @@ specific meta information that is extracted or written in an image.
 The tables listed below give the names of all tags recognized by ExifTool.
 },
     ExifTool2 => q{
-A B<Tag ID> or B<Index> is given in the first column of each table.  A
-B<Tag ID> is the computer-readable equivalent of a tag name, and is the
-identifier that is actually stored in the file.  An B<Index> refers to the
-location of the information, and is used if the information is stored at a
-fixed position in a data block.
+B<Tag ID>, B<Index> or B<Sequence> is given in the first column of each
+table.  A B<Tag ID> is the computer-readable equivalent of a tag name, and
+is the identifier that is actually stored in the file.  An B<Index> refers
+to the location of a value when found at a fixed position within a data
+block, and B<Sequence> gives the order of values for a serial data stream.
 
 A B<Tag Name> is the handle by which the information is accessed in
 ExifTool.  In some instances, more than one name may correspond to a single
@@ -94,16 +95,17 @@ of values written, or the number of characters in a fixed-length string
 
 An asterisk (C<*>) after an entry in the B<Writable> column indicates a
 "protected" tag which is not writable directly, but is set via a Composite
-tag.  A tilde (C<~>) indicates a tag this is only writable when print
-conversion is disabled (by setting PrintConv to 0, or using the -n option).
-A slash (C</>) indicates an "avoided" tag that is not created unless the
-group is specified (due to name conflicts with other tags).  An exclamation
-point (C<!>) indicates a tag that is considered unsafe to write under normal
-circumstances.  These "unsafe" tags are not set when calling
+or Extra tag.  A tilde (C<~>) indicates a tag this is only writable when
+print conversion is disabled (by setting PrintConv to 0, or using the -n
+option). A slash (C</>) indicates an "avoided" tag that is not created
+unless the group is specified (due to name conflicts with other tags).  An
+exclamation point (C<!>) indicates a tag that is considered unsafe to write
+under normal circumstances.  These "unsafe" tags are not set when calling
 SetNewValuesFromFile() or when using the exiftool -TagsFromFile option
 unless specified explicitly, and care should be taken when editing them
 manually since they may affect the way an image is rendered.  A plus sign
-(C<+>) indicates a "list" tag which supports multiple instances.
+(C<+>) indicates a "list" tag which supports multiple values while allowing
+individual values to be added and deleted.
 
 The HTML version of these tables also list possible B<Values> for
 discrete-valued tags, as well as B<Notes> for some tags.
@@ -118,14 +120,14 @@ question.
     EXIF => q{
 EXIF stands for "Exchangeable Image File Format".  This type of information
 is formatted according to the TIFF specification, and may be found in JPG,
-TIFF, PNG, MIFF and WDP images, as well as many TIFF-based RAW images.
+TIFF, PNG, MIFF and HDP images, as well as many TIFF-based RAW images.
 
 The EXIF meta information is organized into different Image File Directories
 (IFD's) within an image.  The names of these IFD's correspond to the
 ExifTool family 1 group names.  When writing EXIF information, the default
 B<Group> listed below is used unless another group is specified.
 
-Also listed in the table below are TIFF, DNG, WDP and other tags which are
+Also listed in the table below are TIFF, DNG, HDP and other tags which are
 not part of the EXIF specification, but may co-exist with EXIF tags in some
 images.
 },
@@ -150,7 +152,8 @@ API).
 XMP stands for "Extensible Metadata Platform", an XML/RDF-based metadata
 format which is being pushed by Adobe.  Information in this format can be
 embedded in many different image file types including JPG, JP2, TIFF, PNG,
-MIFF, PS, PDF, PSD and DNG.
+MIFF, PS, PDF, PSD and DNG, as well as audio file formats supporting ID3v2
+information.
 
 The XMP B<Tag ID>'s aren't listed because in most cases they are identical
 to the B<Tag Name>.
@@ -159,10 +162,18 @@ All XMP information is stored as character strings.  The B<Writable> column
 specifies the information format:  C<integer> is a string of digits
 (possibly beginning with a '+' or '-'), C<real> is a floating point number,
 C<rational> is two C<integer> strings separated by a '/' character, C<date>
-is a date/time string in the format "YYYY:MM:DD HH:MM:SS[+/-HH:MM]",
+is a date/time string in the format "YYYY:MM:DD HH:MM:SS[.SS][+/-HH:MM]",
 C<boolean> is either "True" or "False", and C<lang-alt> is a list of string
-alternatives in different languages. Currently, ExifTool only writes the
-"x-default" language in C<lang-alt> lists.
+alternatives in different languages.
+
+Individual languages for C<lang-alt> tags are accessed by suffixing the tag
+name with a '-', followed by an RFC 3066 language code (ie. "XMP:Title-fr",
+or "Rights-en-US").  A C<lang-alt> tag with no language code accesses the
+"x-default" language, but causes other languages to be deleted when writing.
+The "x-default" language code may be specified when writing a new value to
+write only the default language, but note that all languages are still
+deleted if "x-default" tag is deleted.  When reading, "x-default" is not
+specified.
 
 The XMP tags are organized according to schema B<Namespace> in the following
 tables.  Note that a few of the longer namespace prefixes given below have
@@ -258,6 +269,10 @@ The following tags are decoded in unsupported maker notes.  Use the Unknown
 The tags listed in the PDF tables below are those which are used by ExifTool
 to extract meta information, but they are only a small fraction of the total
 number of available PDF tags.
+
+When writing PDF files, ExifTool uses an increment update.  This has an
+advantage that the original PDF can be easily recovered by deleting the
+C<PDF-update> pseudo-group (with C<-PDF-update:all=> on the command line).
 },
     DNG => q{
 The main DNG tags are found in the EXIF table.  The tables below define only
@@ -302,7 +317,7 @@ L<Image::ExifTool::BuildTagLookup|Image::ExifTool::BuildTagLookup>.
 
 ~head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -391,13 +406,18 @@ sub new
         # save all tag names
         my ($tagID, $binaryTable, $noID, $isIPTC);
         $isIPTC = 1 if $$table{WRITE_PROC} and $$table{WRITE_PROC} eq \&Image::ExifTool::IPTC::WriteIPTC;
-        $noID = 1 if $short =~ /^(Composite|XMP|Extra|Shortcuts|ASF.*)$/;
-        if ($table->{PROCESS_PROC} and
-           ($table->{PROCESS_PROC} eq \&Image::ExifTool::ProcessBinaryData or
-            $table->{PROCESS_PROC} eq \&Image::ExifTool::Real::ProcessSerialData))
+        $noID = 1 if $short =~ /^(Composite|XMP|Extra|Shortcuts|ASF.*)$/ or $table->{VARS}->{NO_ID};
+        if (($table->{VARS} and $table->{VARS}->{INDEX}) or
+            ($table->{PROCESS_PROC} and
+             $table->{PROCESS_PROC} eq \&Image::ExifTool::ProcessBinaryData))
         {
             $binaryTable = 1;
             $id{$tableName} = 'Index';
+        } elsif ($table->{PROCESS_PROC} and
+            $table->{PROCESS_PROC} eq \&Image::ExifTool::Canon::ProcessSerialData)
+        {
+            $binaryTable = 1;
+            $id{$tableName} = 'Sequence';
         } elsif ($isIPTC and $$table{PROCESS_PROC}) { #only the main IPTC table has a PROCESS_PROC
             $id{$tableName} = 'Record';
         } elsif (not $noID) {
@@ -467,6 +487,7 @@ TagID:  foreach $tagID (@keys) {
                     }
                 }
                 my $printConv = $$tagInfo{PrintConv};
+                push @values, sprintf('[Mask 0x%x]',$$tagInfo{Mask}) if $$tagInfo{Mask};
                 if (ref($printConv) =~ /^(HASH|ARRAY)$/) {
                     my (@printConvList, @indexList, $index);
                     if (ref $printConv eq 'ARRAY') {
@@ -479,7 +500,25 @@ TagID:  foreach $tagID (@keys) {
                         $index = shift @indexList;
                     }
                     while (defined $printConv) {
-                        push @values, "[Value $index]" if defined $index;
+                        if (defined $index) {
+                            # (print indices of original values if reorganized)
+                            my $s = '';
+                            my $idx = $$tagInfo{Relist} ? $tagInfo->{Relist}->[$index] : $index;
+                            if (ref $idx) {
+                                $s = 's' if @$idx > 1;
+                                ($idx = join ', ', @$idx) =~ s/(.*),/$1 and/;
+                            } elsif (not $$tagInfo{Relist}) {
+                                while (@printConvList and $printConv eq $printConvList[0]) {
+                                    shift @printConvList;
+                                    $index = shift @indexList;
+                                }
+                                if ($idx != $index) {
+                                    $idx = "$idx-$index";
+                                    $s = 's';
+                                }
+                            }
+                            push @values, "[Value$s $idx]";
+                        }
                         if ($$tagInfo{SeparateTable}) {
                             $subdir = 1;
                             my $s = $$tagInfo{SeparateTable};
@@ -498,14 +537,18 @@ TagID:  foreach $tagID (@keys) {
                                 next if $_ eq '';
                                 $_ eq 'BITMASK' and $bits = $$printConv{$_}, next;
                                 my $index;
-                                if ($$tagInfo{PrintHex}) {
+                                if ($$tagInfo{PrintHex} or $$printConv{BITMASK}) {
                                     $index = sprintf('0x%x',$_);
                                 } elsif (/^[-+]?\d+$/) {
                                     $index = $_;
                                 } else {
-                                    # ignore unprintable values
-                                    next if /[\x00-\x1f\x80-\xff]/;
-                                    $index = "'$_'";
+                                    $index = $_;
+                                    # translate unprintable values
+                                    if ($index =~ s/([\x00-\x1f\x80-\xff])/sprintf("\\x%.2x",ord $1)/eg) {
+                                        $index = qq{"$index"};
+                                    } else {
+                                        $index = qq{'$index'};
+                                    }
                                 }
                                 push @values, "$index = " . $$printConv{$_};
                             }
@@ -521,7 +564,9 @@ TagID:  foreach $tagID (@keys) {
                         $index = shift @indexList;
                     }
                 } elsif ($printConv and $printConv =~ /DecodeBits\(\$val,\s*(\{.*\})\s*\)/s) {
+                    $$self{CameraModel} = '';   # needed for Nikon ShootingMode
                     my $bits = eval $1;
+                    delete $$self{CameraModel};
                     if ($@) {
                         warn $@;
                     } else {
@@ -644,8 +689,8 @@ TagID:  foreach $tagID (@keys) {
 # save TagName information
 #
             my $tagIDstr;
-            if ($tagID =~ /^\d+$/) {
-                if ($binaryTable or $isIPTC or ($short =~ /^CanonCustom/ and $tagID < 256)) {
+            if ($tagID =~ /^\d+(\.\d+)?$/) {
+                if ($1 or $binaryTable or $isIPTC or ($short =~ /^CanonCustom/ and $tagID < 256)) {
                     $tagIDstr = $tagID;
                 } else {
                     $tagIDstr = sprintf("0x%.4x",$tagID);
@@ -813,8 +858,8 @@ sub WriteTagLookup($$)
 sub NumbersFirst
 {
     my $rtnVal;
-    my $bNum = ($b =~ /^-?[0-9]+$/);
-    if ($a =~ /^-?[0-9]+$/) {
+    my $bNum = ($b =~ /^-?[0-9]+(\.\d*)?$/);
+    if ($a =~ /^-?[0-9]+(\.\d*)?$/) {
         $rtnVal = ($bNum ? $a <=> $b : -1);
     } elsif ($bNum) {
         $rtnVal = 1;
@@ -850,6 +895,7 @@ sub Doc2Html($)
     $doc =~ s/\n\n/<\/p>\n\n<p>/g;
     $doc =~ s/B&lt;(.*?)&gt;/<b>$1<\/b>/sg;
     $doc =~ s/C&lt;(.*?)&gt;/<code>$1<\/code>/sg;
+    $doc =~ s/L&lt;(.*?)&gt;/<a href="$1">$1<\/a>/sg;
     return $doc;
 }
 
@@ -1279,7 +1325,7 @@ sub WriteTagNames($$)
                 $wTag = $longTag;
                 warn "Notice: Long tags in $tableName table\n";
             }
-        } elsif ($short =~ /^(Extra|XMP|ASF)/) {
+        } elsif ($short !~ /^(Composite|Shortcuts)/) {
             $wTag += 9;
             $hid = '';
         } else {
@@ -1309,8 +1355,10 @@ sub WriteTagNames($$)
         $line .= sprintf " %-${wGrp}s", 'Group' if $showGrp;
         $line .= ' Writable';
         print PODFILE $line;
+        $line =~ s/Sequence /Sequence\t/;   # don't want an underline after 'Sequence'
         $line =~ s/\S/-/g;
         $line =~ s/- -/---/g;
+        $line =~ tr/\t/ /;
         print PODFILE $line,"\n";
         close HTMLFILE;
         OpenHtmlFile($htmldir, $short) or $success = 0;
@@ -1329,9 +1377,9 @@ sub WriteTagNames($$)
             my ($align, $idStr, $w);
             if (not $id) {
                 $idStr = '  ';
-            } elsif ($tagIDstr =~ /^\d+$/) {
+            } elsif ($tagIDstr =~ /^\d+(\.\d+)?$/) {
                 $w = $wID - 3;
-                $idStr = sprintf "  %${w}d    ", $tagIDstr;
+                $idStr = sprintf "  %${w}g    ", $tagIDstr;
                 $align = " class=r";
             } else {
                 $tagIDstr =~ s/^'$prefix/'/ if $prefix;
@@ -1381,6 +1429,7 @@ sub WriteTagNames($$)
                 printf PODFILE " %-${w}s", shift(@reqs) || '';
             }
             printf PODFILE " $wrStr\n";
+            my $numTags = scalar @$tagNames;
             my $n = 0;
             while (@tags or @reqs or @vals) {
                 my $more = (@tags or @reqs);
@@ -1433,8 +1482,8 @@ sub WriteTagNames($$)
                 if ($isSubdir) {
                     my $smallNote;
                     foreach (@$values) {
-                        if (/^\(/) {
-                            $smallNote = 1 if $n <= 1;
+                        if (/^[[(]/) {
+                            $smallNote = 1 if $numTags < 2;
                             push @values, ($smallNote ? $noteFontSmall : $noteFont) . "$_</span>";
                             next;
                         }
@@ -1508,9 +1557,27 @@ documentation.
 
   $ok = $builder->WriteTagNames('lib/Image/ExifTool/TagNames.pod','html');
 
+=head1 MEMBER VARIABLES
+
+=over 4
+
+=item PRESERVE_DATE
+
+Flag to preserve "Last revised" date in HTML files.  Set before calling
+WriteTagNames().
+
+=item COUNT
+
+Reference to hash containing counting statistics.  Keys are the
+descriptions, and values are the numerical counts.  Valid after
+BuildTagLookup object is created, but additional statistics are added by
+WriteTagNames().
+
+=back
+
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
