@@ -15,7 +15,7 @@ use Image::ExifTool::Exif;
 
 sub ProcessUnknown($$$);
 
-$VERSION = '1.24';
+$VERSION = '1.37';
 
 my $debug;          # set to 1 to enabled debugging code
 
@@ -30,7 +30,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteCanon',
         # (starts with an IFD)
-        Condition => '$self->{CameraMake} =~ /^Canon/',
+        Condition => '$$self{CameraMake} =~ /^Canon/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Canon::Main',
             ByteOrder => 'Unknown',
@@ -41,7 +41,7 @@ my $debug;          # set to 1 to enabled debugging code
         # do negative lookahead assertion just to get tags
         # in a nice order for documentation
         # (starts with an IFD)
-        Condition => '$self->{CameraMake}=~/^CASIO(?! COMPUTER CO.,LTD)/',
+        Condition => '$$self{CameraMake}=~/^CASIO(?! COMPUTER CO.,LTD)/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Casio::Main',
             ByteOrder => 'Unknown',
@@ -50,7 +50,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteCasio2',
         # (starts with "QVC\0")
-        Condition => '$self->{CameraMake}=~/^CASIO COMPUTER CO.,LTD/',
+        Condition => '$$self{CameraMake}=~/^CASIO COMPUTER CO.,LTD/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Casio::Type2',
             Start => '$valuePtr + 6',
@@ -61,21 +61,64 @@ my $debug;          # set to 1 to enabled debugging code
         # The Fuji programmers really botched this one up,
         # but with a bit of work we can still read this directory
         Name => 'MakerNoteFujiFilm',
-        # (starts with "FUJIFILM")
-        Condition => '$self->{CameraMake} =~ /^FUJIFILM/',
+        # (starts with "FUJIFILM" -- also used by some Leica, Minolta and Sharp models)
+        Condition => '$$valPt =~ /^FUJIFILM/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::FujiFilm::Main',
             # there is an 8-byte maker tag (FUJIFILM) we must skip over
             OffsetPt => '$valuePtr+8',
-            ByteOrder => 'LittleEndian',
             # the pointers are relative to the subdirectory start
             # (before adding the offsetPt) - PH
             Base => '$start',
+            ByteOrder => 'LittleEndian',
+        },
+    },
+    {
+        Name => 'MakerNoteHP',  # PhotoSmart 720 (also Vivitar 3705, 3705B and 3715)
+        Condition => '$$valPt =~ /^(Hewlett-Packard|Vivitar)/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::HP::Main',
+            ProcessProc => \&ProcessUnknown,
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNoteHP2', # PhotoSmart E427
+        Condition => q{
+            $$self{CameraMake} =~ /^Hewlett-Packard/ and
+            $$valPt !~ /^.{8}Hewlett-Packard/s and
+            $$valPt !~ /^IIII/ and $$valPt =~ /^\d{3}.\0/s
+        },
+        NotIFD => 1,
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::HP::Type2',
+            Start => '$valuePtr',
+            ByteOrder => 'LittleEndian',
+        },
+    },
+    {
+        Name => 'MakerNoteHP4', # PhotoSmart M627
+        Condition => '$$valPt =~ /^IIII\x04\0/',
+        NotIFD => 1,
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::HP::Type4',
+            Start => '$valuePtr',
+            ByteOrder => 'LittleEndian',
+        },
+    },
+    {
+        Name => 'MakerNoteHP6', # PhotoSmart M425, M525 and M527
+        Condition => '$$valPt =~ /^IIII\x06\0/',
+        NotIFD => 1,
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::HP::Type6',
+            Start => '$valuePtr',
+            ByteOrder => 'LittleEndian',
         },
     },
     {
         Name => 'MakerNoteJVC',
-        Condition => '$self->{CameraMake}=~/^JVC/ and $$valPt=~/^JVC /',
+        Condition => '$$valPt=~/^JVC /',
         SubDirectory => {
             TagTable => 'Image::ExifTool::JVC::Main',
             Start => '$valuePtr + 4',
@@ -84,14 +127,16 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteJVCText',
-        Condition => '$self->{CameraMake}=~/^(JVC|Victor)/ and $$valPt=~/^VER:/',
+        Condition => '$$self{CameraMake}=~/^(JVC|Victor)/ and $$valPt=~/^VER:/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::JVC::Text',
         },
     },
     {
         Name => 'MakerNoteKodak1a',
-        Condition => '$self->{CameraMake}=~/^EASTMAN KODAK/ and $$valPt=~/^KDK INFO/',
+        Condition => '$$self{CameraMake}=~/^EASTMAN KODAK/ and $$valPt=~/^KDK INFO/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Main',
             Start => '$valuePtr + 8',
@@ -100,7 +145,8 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteKodak1b',
-        Condition => '$self->{CameraMake}=~/^EASTMAN KODAK/ and $$valPt=~/^KDK/',
+        Condition => '$$self{CameraMake}=~/^EASTMAN KODAK/ and $$valPt=~/^KDK/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Main',
             Start => '$valuePtr + 8',
@@ -108,8 +154,13 @@ my $debug;          # set to 1 to enabled debugging code
         },
     },
     {
+        # used by various Kodak, HP, Pentax and Minolta models
         Name => 'MakerNoteKodak2',
-        Condition => '$self->{CameraMake}=~/^EASTMAN KODAK/i and $$valPt=~/^.{8}Eastman/s',
+        Condition => q{
+            $$valPt =~ /^.{8}Eastman Kodak/s or
+            $$valPt =~ /^\x01\0[\0\x01]\0\0\0\x04\0[a-zA-Z]{4}/
+        },
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type2',
             ByteOrder => 'BigEndian',
@@ -119,7 +170,8 @@ my $debug;          # set to 1 to enabled debugging code
         # not much to key on here, but we know the
         # upper byte of the year should be 0x07:
         Name => 'MakerNoteKodak3',
-        Condition => '$self->{CameraMake}=~/^EASTMAN KODAK/ and $$valPt=~/^.{12}\x07/s',
+        Condition => '$$self{CameraMake}=~/^EASTMAN KODAK/ and $$valPt=~/^.{12}\x07/s',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type3',
             ByteOrder => 'BigEndian',
@@ -127,7 +179,8 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteKodak4',
-        Condition => '$self->{CameraMake}=~/^Eastman Kodak/ and $$valPt=~/^.{41}JPG/s',
+        Condition => '$$self{CameraMake}=~/^Eastman Kodak/ and $$valPt=~/^.{41}JPG/s',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type4',
             ByteOrder => 'BigEndian',
@@ -136,11 +189,12 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteKodak5',
         Condition => q{
-            $self->{CameraMake}=~/^EASTMAN KODAK/ and
-            ($self->{CameraModel}=~/CX(4200|4230|4300|4310|6200|6230)/ or
+            $$self{CameraMake}=~/^EASTMAN KODAK/ and
+            ($$self{CameraModel}=~/CX(4200|4230|4300|4310|6200|6230)/ or
             # try to pick up similar models we haven't tested yet
             $$valPt=~/^\0(\x1a\x18|\x3a\x08|\x59\xf8|\x14\x80)\0/)
         },
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type5',
             ByteOrder => 'BigEndian',
@@ -149,9 +203,10 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteKodak6a',
         Condition => q{
-            $self->{CameraMake}=~/^EASTMAN KODAK/ and
-            $self->{CameraModel}=~/DX3215/
+            $$self{CameraMake}=~/^EASTMAN KODAK/ and
+            $$self{CameraModel}=~/DX3215/
         },
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type6',
             ByteOrder => 'BigEndian',
@@ -160,17 +215,49 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteKodak6b',
         Condition => q{
-            $self->{CameraMake}=~/^EASTMAN KODAK/ and
-            $self->{CameraModel}=~/DX3700/
+            $$self{CameraMake}=~/^EASTMAN KODAK/ and
+            $$self{CameraModel}=~/DX3700/
         },
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Type6',
             ByteOrder => 'LittleEndian',
         },
     },
+    {
+        Name => 'MakerNoteKodak7',
+        # look for something that looks like a serial number
+        # (confirmed serial numbers have the format KXXXX########, but we also
+        #  accept other strings from sample images that may be serial numbers)
+        Condition => q{
+            $$self{CameraMake}=~/Kodak/i and
+            $$valPt =~ /^[CK][A-Z\d]{3} ?[A-Z\d]{1,2}\d{2}[A-Z\d]\d{4}[ \0]/
+        },
+        NotIFD => 1,
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Kodak::Type7',
+            ByteOrder => 'LittleEndian',
+        },
+    },
+    {
+        Name => 'MakerNoteKodakIFD',
+        # look for reasonable number of entries and check
+        # format and count of first IFD entry
+        Condition => q{
+            $$self{CameraMake}=~/Kodak/i and
+            ($$valPt =~ /^\0[\x02-\x7f]..\0[\x01-\x0c]\0\0/s or
+             $$valPt =~ /^[\x02-\x7f]\0..[\x01-\x0c]\0..\0\0/s)
+        },
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Kodak::IFD',
+            ProcessProc => \&ProcessUnknown,
+            ByteOrder => 'Unknown',
+        },
+    },
     {
         Name => 'MakerNoteKodakUnknown',
-        Condition => '$self->{CameraMake}=~/Kodak/i',
+        Condition => '$$self{CameraMake}=~/Kodak/i and $$valPt!~/^AOC\0/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Kodak::Unknown',
             ByteOrder => 'BigEndian',
@@ -179,7 +266,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteKyocera',
         # (starts with "KYOCERA")
-        Condition => '$self->{CameraMake}=~/^KYOCERA/',
+        Condition => '$$self{CameraMake}=~/^KYOCERA/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Unknown::Main',
             Start => '$valuePtr + 22',
@@ -191,8 +278,8 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteMinolta',
         Condition => q{
-            $self->{CameraMake}=~/^(Konica Minolta|Minolta)/i and
-            $$valPt !~ /^(MINOL|CAMER|FUJIFILM|MLY0|KC|\+M\+M|\xd7)/
+            $$self{CameraMake}=~/^(Konica Minolta|Minolta)/i and
+            $$valPt !~ /^(MINOL|CAMER|MLY0|KC|\+M\+M|\xd7)/
         },
         SubDirectory => {
             TagTable => 'Image::ExifTool::Minolta::Main',
@@ -200,12 +287,10 @@ my $debug;          # set to 1 to enabled debugging code
         },
     },
     {
-        # the DiMAGE E323 (MINOL) and E500 (CAMER)
+        # the DiMAGE E323 (MINOL) and E500 (CAMER), and some models
+        # of Mustek, Pentax, Ricoh and Vivitar (CAMER).
         Name => 'MakerNoteMinolta2',
-        Condition => q{
-            $self->{CameraMake} =~ /^(Konica Minolta|Minolta)/i and
-            $$valPt =~ /^(MINOL|CAMER)/
-        },
+        Condition => '$$valPt =~ /^(MINOL|CAMER)\0/ and $$self{OlympusCAMER} = 1',
         SubDirectory => {
             # these models use Olympus tags in the range 0x200-0x221 plus 0xf00
             TagTable => 'Image::ExifTool::Olympus::Main',
@@ -213,45 +298,32 @@ my $debug;          # set to 1 to enabled debugging code
             ByteOrder => 'Unknown',
         },
     },
-    {
-        # the DiMAGE F200
-        Name => 'MakerNoteMinolta3',
-        Condition => q{
-            $self->{CameraMake} =~ /^(Konica Minolta|Minolta)/i and
-            $$valPt =~ /^FUJIFILM/
-        },
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::FujiFilm::Main',
-            OffsetPt => '$valuePtr+8',
-            ByteOrder => 'LittleEndian',
-            Base => '$start',
-        },
-    },
     {
         # /^MLY0/ - DiMAGE G400, G500, G530, G600
         # /^KC/   - Revio KD-420Z, DiMAGE E203
         # /^+M+M/ - DiMAGE E201
         # /^\xd7/ - DiMAGE RD3000
-        Name => 'MakerNoteMinolta4',
-        Condition => '$self->{CameraMake} =~ /^(Konica Minolta|Minolta)/i',
+        Name => 'MakerNoteMinolta3',
+        Condition => '$$self{CameraMake} =~ /^(Konica Minolta|Minolta)/i',
         Notes => 'not EXIF-based',
         Binary => 1,
+        NotIFD => 1,
     },
     {
         # this maker notes starts with a standard TIFF header at offset 0x0a
         Name => 'MakerNoteNikon',
-        Condition => '$self->{CameraMake}=~/^NIKON/i and $$valPt=~/^Nikon\x00\x02/',
+        Condition => '$$self{CameraMake}=~/^NIKON/i and $$valPt=~/^Nikon\x00\x02/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Nikon::Main',
             Start => '$valuePtr + 18',
-            ByteOrder => 'Unknown',
             Base => '$start - 8',
+            ByteOrder => 'Unknown',
         },
     },
     {
         # older Nikon maker notes
         Name => 'MakerNoteNikon2',
-        Condition => '$self->{CameraMake}=~/^NIKON/ and $$valPt=~/^Nikon\x00\x01/',
+        Condition => '$$self{CameraMake}=~/^NIKON/ and $$valPt=~/^Nikon\x00\x01/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Nikon::Type2',
             Start => '$valuePtr + 8',
@@ -261,7 +333,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         # headerless Nikon maker notes
         Name => 'MakerNoteNikon3',
-        Condition => '$self->{CameraMake}=~/^NIKON/i',
+        Condition => '$$self{CameraMake}=~/^NIKON/i',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Nikon::Main',
             ByteOrder => 'Unknown', # most are little-endian, but D1 is big
@@ -272,17 +344,28 @@ my $debug;          # set to 1 to enabled debugging code
         # (if Make is 'SEIKO EPSON CORP.', starts with "EPSON\0")
         # (if Make is 'OLYMPUS OPTICAL CO.,LTD' or 'OLYMPUS CORPORATION',
         #  starts with "OLYMP\0")
-        Condition => '$self->{CameraMake} =~ /^(OLYMPUS|SEIKO EPSON|AGFA )/i',
+        Condition => '$$valPt =~ /^(OLYMP|EPSON)\0/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Olympus::Main',
             Start => '$valuePtr + 8',
             ByteOrder => 'Unknown',
         },
     },
+    {
+        Name => 'MakerNoteOlympus2',
+        # new Olympus maker notes start with "OLYMPUS\0"
+        Condition => '$$valPt =~ /^OLYMPUS\0/ and $$self{OlympusType2} = 1',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Olympus::Main',
+            Start => '$valuePtr + 12',
+            Base => '$start - 12',
+            ByteOrder => 'Unknown',
+        },
+    },
     {
         Name => 'MakerNoteLeica',
         # (starts with "LEICA\0")
-        Condition => '$self->{CameraMake} =~ /^LEICA/',
+        Condition => '$$self{CameraMake} =~ /^LEICA/',
         SubDirectory => {
             # Leica uses the same format as Panasonic
             TagTable => 'Image::ExifTool::Panasonic::Main',
@@ -293,7 +376,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNotePanasonic',
         # (starts with "Panasonic\0")
-        Condition => '$self->{CameraMake} =~ /^Panasonic/ and $$valPt!~/^MKE/',
+        Condition => '$$self{CameraMake} =~ /^Panasonic/ and $$valPt!~/^MKE/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Main',
             Start => '$valuePtr + 12',
@@ -303,7 +386,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNotePanasonic2',
         # (starts with "Panasonic\0")
-        Condition => '$self->{CameraMake} =~ /^Panasonic/',
+        Condition => '$$self{CameraMake} =~ /^Panasonic/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Panasonic::Type2',
             ByteOrder => 'LittleEndian',
@@ -311,23 +394,62 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNotePentax',
-        # (if Make is 'PENTAX Corporation', starts with "AOC\0" (also Samsung DX-1S))
-        # (if Make is 'Asahi Optical Co.,Ltd', starts with an IFD)
-        Condition => '$self->{CameraMake}=~/^(PENTAX|AOC|Asahi)/ or $$valPt=~/^AOC\0/',
+        # (starts with "AOC\0", but so does MakerNotePentax3)
+        # (also used by some Samsung models)
+        Condition => q{
+            $$valPt=~/^AOC\0/ and
+            $$self{CameraModel} !~ /^PENTAX Optio ?[34]30RS\s*$/
+        },
         SubDirectory => {
             TagTable => 'Image::ExifTool::Pentax::Main',
             # process as Unknown maker notes because the start offset and
             # byte ordering are so variable
             ProcessProc => \&ProcessUnknown,
-            ByteOrder => 'Unknown',
             # offsets can be totally whacky for Pentax maker notes,
             # so attempt to fix the offset base if possible
             FixBase => 1,
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNotePentax2',
+        # (starts with an IFD)
+        # Casio-like maker notes used only by the Optio 330 and 430
+        Condition => '$$self{CameraMake}=~/^Asahi/ and $$valPt!~/^AOC\0/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Pentax::Type2',
+            ProcessProc => \&ProcessUnknown,
+            FixBase => 1,
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNotePentax3',
+        # (starts with "AOC\0", like the more common Pentax maker notes)
+        # Casio maker notes used only by the Optio 330RS and 430RS
+        Condition => '$$self{CameraMake}=~/^Asahi/',
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Casio::Type2',
+            ProcessProc => \&ProcessUnknown,
+            FixBase => 1,
+            ByteOrder => 'Unknown',
+        },
+    },
+    {
+        Name => 'MakerNotePentax4',
+        # (starts with 3 or 4 digits)
+        # HP2-like text-based maker notes used by Optio E20
+        Condition => '$$self{CameraMake}=~/^PENTAX/ and $$valPt=~/^\d{3}/',
+        NotIFD => 1,
+        SubDirectory => {
+            TagTable => 'Image::ExifTool::Pentax::Type4',
+            Start => '$valuePtr',
+            ByteOrder => 'LittleEndian',
         },
     },
     {
         Name => 'MakerNoteRicoh',
-        Condition => '$self->{CameraMake}=~/^RICOH/ and $$valPt=~/^Ricoh/i',
+        Condition => '$$self{CameraMake}=~/^RICOH/ and $$valPt=~/^Ricoh/i',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Ricoh::Main',
             Start => '$valuePtr + 8',
@@ -336,22 +458,23 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteRicohText',
-        Condition => '$self->{CameraMake}=~/^RICOH/',
+        Condition => '$$self{CameraMake}=~/^RICOH/',
+        NotIFD => 1,
         SubDirectory => {
             TagTable => 'Image::ExifTool::Ricoh::Text',
             ByteOrder => 'Unknown',
         },
     },
     {
-        # Samsung PreviewImage
+        # PreviewImage (Samsung Digimax plus some other types of cameras)
         %Image::ExifTool::previewImageTagInfo,
-        Condition => '$self->{CameraMake}=~/^Samsung/ and $self->{CameraModel}=~/^<Digimax/',
+        Condition => '$$valPt =~ /^\xff\xd8\xff\xdb/ and $$self{CameraMake}!~/(SONY|SANYO|SIGMA)/i',
         Notes => 'Samsung preview image',
     },
     {
         Name => 'MakerNoteSanyo',
         # (starts with "SANYO\0")
-        Condition => '$self->{CameraMake}=~/^SANYO/ and $self->{CameraModel} !~ /^(C4|J\d|S\d)\b/',
+        Condition => '$$self{CameraMake}=~/^SANYO/ and $$self{CameraModel}!~/^(C4|J\d|S\d)\b/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sanyo::Main',
             Validate => '$val =~ /^SANYO/',
@@ -362,19 +485,19 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteSanyoC4',
         # The C4 offsets are wrong by 12, so they must be fixed
-        Condition => '$self->{CameraMake}=~/^SANYO/ and $self->{CameraModel} =~ /^C4\b/',
+        Condition => '$$self{CameraMake}=~/^SANYO/ and $$self{CameraModel}=~/^C4\b/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sanyo::Main',
             Validate => '$val =~ /^SANYO/',
             Start => '$valuePtr + 8',
-            ByteOrder => 'Unknown',
             FixBase => 1,
+            ByteOrder => 'Unknown',
         },
     },
     {
         Name => 'MakerNoteSanyoPatch',
         # The J1, J2, J4, S1, S3 and S4 offsets are completely screwy
-        Condition => '$self->{CameraMake}=~/^SANYO/',
+        Condition => '$$self{CameraMake}=~/^SANYO/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sanyo::Main',
             Validate => '$val =~ /^SANYO/',
@@ -386,7 +509,7 @@ my $debug;          # set to 1 to enabled debugging code
     {
         Name => 'MakerNoteSigma',
         # (starts with "SIGMA\0")
-        Condition => '$self->{CameraMake}=~/^(SIGMA|FOVEON)/',
+        Condition => '$$self{CameraMake}=~/^(SIGMA|FOVEON)/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sigma::Main',
             Validate => '$val =~ /^(SIGMA|FOVEON)/',
@@ -396,30 +519,28 @@ my $debug;          # set to 1 to enabled debugging code
     },
     {
         Name => 'MakerNoteSony',
-        # (starts with "SONY DSC \0")
-        Condition => '$self->{CameraMake}=~/^SONY/ and $self->{TIFF_TYPE}!~/^(SRF|SR2|DNG)$/',
+        # (starts with "SONY DSC \0" or "SONY CAM \0")
+        Condition => '$$self{CameraMake}=~/^SONY/ and $$valPt=~/^SONY (DSC|CAM)/',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Sony::Main',
-            # validate the maker note because this is sometimes garbage
-            Validate => 'defined($val) and $val =~ /^SONY DSC/',
             Start => '$valuePtr + 12',
             ByteOrder => 'Unknown',
         },
     },
     {
-        Name => 'MakerNoteSonySRF',
-        Condition => '$self->{CameraMake}=~/^SONY/ and $$valPt =~ /^\x01\x00/',
+        Name => 'MakerNoteSony2', # used in SR2 and ARW images
+        Condition => '$$self{CameraMake}=~/^SONY/ and $$valPt!~/^\x01\x00/',
         SubDirectory => {
-            TagTable => 'Image::ExifTool::Sony::SRF',
+            TagTable => 'Image::ExifTool::Sony::Main',
             Start => '$valuePtr',
             ByteOrder => 'Unknown',
         },
     },
     {
-        Name => 'MakerNoteSonySR2',
-        Condition => '$self->{CameraMake}=~/^SONY/',
+        Name => 'MakerNoteSonySRF',
+        Condition => '$$self{CameraMake}=~/^SONY/',
         SubDirectory => {
-            TagTable => 'Image::ExifTool::Sony::Main',
+            TagTable => 'Image::ExifTool::Sony::SRF',
             Start => '$valuePtr',
             ByteOrder => 'Unknown',
         },
@@ -554,29 +675,59 @@ sub FixBase($$)
 {
     local $_;
     my ($exifTool, $dirInfo) = @_;
-    return 0 if $$dirInfo{FixOffsets}; # don't fix base if fixing offsets individually
+    # don't fix base if fixing offsets individually or if we don't want to fix them
+    return 0 if $$dirInfo{FixOffsets} or $$dirInfo{NoFixBase};
 
     my $dataPt = $$dirInfo{DataPt};
+    my $dataPos = $$dirInfo{DataPos};
     my $dirStart = $$dirInfo{DirStart} || 0;
     my $entryBased = $$dirInfo{EntryBased};
-
-    # get hash of value block positions
-    my ($valBlock, $valBlkAdj) = GetValueBlocks($dataPt, $dirStart);
-    return 0 unless %$valBlock;
+    my $dirName = $$dirInfo{DirName};
+    my $fixBase = $exifTool->Options('FixBase');
+    my $setBase = (defined $fixBase and $fixBase ne '') ? 1 : 0;
+    my ($fix, $fixedBy);
+#
+# handle special case of Canon maker notes with TIFF trailer containing original offset
+#
+    if ($$exifTool{CameraMake} =~ /^Canon/ and $$dirInfo{DirLen} > 8) {
+        my $trailerPos = $dirStart + $$dirInfo{DirLen} - 8;
+        my $trailer = substr($$dataPt, $trailerPos, 8);
+        if ($trailer =~ /^(II\x2a\0|MM\0\x2a)/ and  # check for TIFF trailer
+            substr($trailer,0,2) eq GetByteOrder()) # validate byte ordering
+        {
+            if ($$exifTool{HTML_DUMP}) {
+                my $filePos = ($$dirInfo{Base} || 0) + $dataPos + $trailerPos;
+                $exifTool->HtmlDump($filePos, 8, '[Canon MakerNote trailer]')
+            }
+            if ($setBase) {
+                $fix = $fixBase;
+            } else {
+                my $oldOffset = Get32u(\$trailer, 4);
+                my $newOffset = $dirStart + $dataPos;
+                $fix = $newOffset - $oldOffset;
+                return 0 unless $fix;
+            }
+            $exifTool->Warn("Adjusted $dirName base by $fix", 1);
+            $$dirInfo{FixedBy} = $fix;
+            $$dirInfo{Base} += $fix;
+            $$dirInfo{DataPos} -= $fix;
+            return $fix;
+        }
+    }
 #
 # analyze value offsets to see if they need fixing.  The first task is to
 # determine the minimum valid offset used (this is tricky, because we have
 # to weed out bad offsets written by some cameras)
 #
-    my $dataPos = $$dirInfo{DataPos};
+    # get hash of value block positions
+    my ($valBlock, $valBlkAdj) = GetValueBlocks($dataPt, $dirStart);
+    return 0 unless %$valBlock;
+
     my $ifdLen = 2 + 12 * Get16u($$dirInfo{DataPt}, $dirStart);
     my $ifdEnd = $dirStart + $ifdLen;
     my ($relative, @offsets) = GetMakerNoteOffset($exifTool);
     my $makeDiff = $offsets[0];
-    my $fixBase = $exifTool->Options('FixBase');
-    my $setBase = (defined $fixBase and $fixBase ne '') ? 1 : 0;
     my $verbose = $exifTool->Options('Verbose');
-    my $dirName = $$dirInfo{DirName};
     my ($diff, $shift);
 
     # calculate expected minimum value offset
@@ -626,7 +777,7 @@ sub FixBase($$)
         # calculate difference from normal
         $diff = ($minPt - $dataPos) - $ifdEnd;
         $shift = 0;
-        $countOverlap and $exifTool->Warn("Overlapping $dirName values");
+        $countOverlap and $exifTool->Warn("Overlapping $dirName values",1);
         if ($entryBased) {
             $exifTool->Warn("$dirName offsets do NOT look entry-based");
             undef $entryBased;
@@ -673,8 +824,7 @@ sub FixBase($$)
 #
     # use default padding of 4 bytes unless already specified
     $makeDiff = 4 unless defined $makeDiff;
-    my $fix = $makeDiff - $diff;    # assume standard diff for this make
-    my $fixedBy;
+    $fix = $makeDiff - $diff;   # assume standard diff for this make
 
     if ($$dirInfo{FixBase}) {
         # set flag if offsets are relative (base is at or above directory start)
@@ -830,13 +980,16 @@ IFD_TRY: for ($offset=$firstTry; $offset<=$lastTry; $offset+=2) {
                 # allow everything to be zero if not first entry
                 # because some manufacturers pad with null entries
                 next unless $format or $count or $index == 0;
+                # patch for Canon EOS 40D firmware 1.0.4 bug: allow zero format for last entry
+                next if $format==0 and $index==$num-1 and $$exifTool{CameraModel}=~/EOS 40D/;
                 # (would like to verify tag ID, but some manufactures don't
                 #  sort entries in order of tag ID so we don't have much of
                 #  a handle to verify this field)
                 # verify format
                 next IFD_TRY if $format < 1 or $format > 13;
-                # count must be reasonable
-                next IFD_TRY if $count == 0 or $count & 0xff000000;
+                # count must be reasonable (can't test for zero count because
+                # cameras like the 1DmkIII use this value)
+                next IFD_TRY if $count & 0xff000000;
             }
             $$dirInfo{DirStart} += $offset;    # update directory start
             $$dirInfo{DirLen} -= $offset;
@@ -858,6 +1011,7 @@ sub ProcessUnknown($$$)
 
     my $loc = LocateIFD($exifTool, $dirInfo);
     if (defined $loc) {
+        $exifTool->{UnknownByteOrder} = GetByteOrder();
         if ($exifTool->Options('Verbose') > 1) {
             my $out = $exifTool->Options('TextOut');
             my $indent = $exifTool->{INDENT};
@@ -867,6 +1021,7 @@ sub ProcessUnknown($$$)
         }
         $success = Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
     } else {
+        $exifTool->{UnknownByteOrder} = ''; # indicates we tried but didn't set byte order
         $exifTool->Warn("Unrecognized $$dirInfo{DirName}");
     }
     return $success;
@@ -892,7 +1047,7 @@ maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
