@@ -28,7 +28,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.15';
+$VERSION = '1.16';
 
 sub ProcessPNG_tEXt($$$);
 sub ProcessPNG_iTXt($$$);
@@ -161,7 +161,7 @@ $Image::ExifTool::PNG::colorType = -1;
             return pack('nC5', @a);
         },
         PrintConv => '$self->ConvertDateTime($val)',
-        PrintConvInv => '$val',
+        PrintConvInv => '$self->InverseDateTime($val)',
     },
     tRNS => {
         Name => 'Transparency',
@@ -271,11 +271,12 @@ listed below are the only ones that can be written (unless new user-defined
 tags are added via the configuration file), however ExifTool will extract
 any other TextualData tags that are found.
 
-The information for the TextualData tags may be stored as tEXt, zTXt or iTXt
-chunks in the PNG image.  ExifTool will read and edit tags in their original
-form, but tEXt chunks are written by default when creating new tags.
-Compressed zTXt chunks are written only if Compress::Zlib is available, and
-only for profile information or when the -z (Compress) option is specified.
+Data for the TextualData tags may be stored as tEXt, zTXt or iTXt chunks in
+the PNG image.  ExifTool will read and edit tags in their original form, but
+new string tags are created as uncompressed tEXt by default, or as
+compressed zTXt if the -z (Compress) option is used and Compress::Zlib is
+available. Raw profile information is always created as compressed zTXt if
+Compress::Zlib is available.
 
 Some of the tags below are not registered as part of the PNG specification,
 but are included here because they are generated by other software such as
@@ -548,7 +549,7 @@ sub FoundPNG($$$$;$$)
         ($name = $tag) =~ s/\s+(.)/\u$1/g;   # remove white space from tag name
         $tagInfo = { Name => $name };
         # make unknown profiles binary data type
-        $$tagInfo{ValueConv} = '\$val' if $tag =~ /^Raw profile type /;
+        $$tagInfo{Binary} = 1 if $tag =~ /^Raw profile type /;
         Image::ExifTool::AddTagToTable($tagTablePtr, $tag, $tagInfo);
     }
 #
@@ -912,7 +913,7 @@ and JNG (JPEG Network Graphics) images.
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
