@@ -16,15 +16,13 @@ use strict;
 use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.14';
+$VERSION = '1.20';
 
 sub WritePS($$);
-sub CheckPS($$$);
 
 # PostScript tag table
 %Image::ExifTool::PostScript::Main = (
     WRITE_PROC => \&WritePS,
-    CHECK_PROC => \&CheckPS,
     PREFERRED => 1, # always add these tags when writing
     GROUPS => { 2 => 'Image' },
     # Note: Make all of these tags priority 0 since the first one found at
@@ -72,6 +70,13 @@ sub CheckPS($$$);
             TagTable => 'Image::ExifTool::XMP::Main',
         },
     },
+    TIFFPreview => {
+        Binary => 1,
+        Notes => q{
+            not a real tag ID, but used to represent the TIFF preview extracted from DOS
+            EPS images
+        },
+    },
 );
 
 # composite tags
@@ -96,7 +101,7 @@ sub CheckPS($$$);
 );
 
 # add our composite tags
-Image::ExifTool::AddCompositeTags(\%Image::ExifTool::PostScript::Composite);
+Image::ExifTool::AddCompositeTags('Image::ExifTool::PostScript');
 
 #------------------------------------------------------------------------------
 # AutoLoad our writer routines when necessary
@@ -164,6 +169,88 @@ sub SetInputRecordSeparator($)
     return $oldsep;
 }
 
+#------------------------------------------------------------------------------
+# Decode comment from PostScript file
+# Inputs: 0) comment string, 1) RAF ref, 2) reference to lines array
+#         3) optional data reference for extra lines read from file
+# Returns: Decoded comment string (may be an array reference)
+# - handles multi-line comments and escape sequences
+sub DecodeComment($$$;$)
+{
+    my ($val, $raf, $lines, $dataPt) = @_;
+    $val =~ s/\x0d*\x0a*$//;        # remove trailing CR, LF or CR/LF
+    # check for continuation comments
+    for (;;) {
+        unless (@$lines) {
+            my $buff;
+            $raf->ReadLine($buff) or last;
+            my $altnl = $/ eq "\x0d" ? "\x0a" : "\x0d";
+            if ($buff =~ /$altnl/) {
+                # split into separate lines
+                @$lines = split /$altnl/, $buff, -1;
+                # handle case of DOS newline data inside file using Unix newlines
+                @$lines = ( $$lines[0] . $$lines[1] ) if @$lines == 2 and $$lines[1] eq $/;
+            } else {
+                push @$lines, $buff;
+            }
+        }
+        last unless $$lines[0] =~ s/^%%\+//;    # is the next line a continuation?
+        $$dataPt .= "%%+$$lines[0]" if $dataPt; # add to data if necessary
+        $$lines[0] =~ s/\x0d*\x0a*$//;          # remove trailing CR, LF or CR/LF
+        $val .= shift @$lines;
+    }
+    my @vals;
+    # handle bracketed string values
+    if ($val =~ s/^\((.*)\)$/$1/) { # remove brackets if necessary
+        # split into an array of strings if necessary
+        my $nesting = 1;
+        while ($val =~ /(\(|\))/g) {
+            my $bra = $1;
+            my $pos = pos($val) - 2;
+            my $backslashes = 0;
+            while ($pos and substr($val, $pos, 1) eq '\\') {
+                --$pos;
+                ++$backslashes;
+            }
+            next if $backslashes & 0x01;    # escaped if odd number
+            if ($bra eq '(') {
+                ++$nesting;
+            } else {
+                --$nesting;
+                unless ($nesting) {
+                    push @vals, substr($val, 0, pos($val)-1);
+                    $val = substr($val, pos($val));
+                    ++$nesting if $val =~ s/\s*\(//;
+                }
+            }
+        }
+        push @vals, $val;
+        foreach $val (@vals) {
+            # decode escape sequences in bracketed strings
+            # (similar to code in PDF.pm, but without line continuation)
+            while ($val =~ /\\(.)/sg) {
+                my $n = pos($val) - 2;
+                my $c = $1;
+                my $r;
+                if ($c =~ /[0-7]/) {
+                    # get up to 2 more octal digits
+                    $c .= $1 if $val =~ /\G([0-7]{1,2})/g;
+                    # convert octal escape code
+                    $r = chr(oct($c) & 0xff);
+                } else {
+                    # convert escaped characters
+                    ($r = $c) =~ tr/nrtbf/\n\r\t\b\f/;
+                }
+                substr($val, $n, length($c)+1) = $r;
+                # continue search after this character
+                pos($val) = $n + length($r);
+            }
+        }
+        $val = @vals > 1 ? \@vals : $vals[0];
+    }
+    return $val;
+}
+
 #------------------------------------------------------------------------------
 # Extract information from EPS, PS or AI file
 # Inputs: 0) ExifTool object reference, 1) dirInfo reference
@@ -205,10 +292,19 @@ sub ProcessPS($$)
 #
 # extract TIFF information from DOS header
 #
+    my $tagTablePtr = GetTagTable('Image::ExifTool::PostScript::Main');
     if ($dos) {
         my $base = Get32u(\$dos, 16);
         if ($base) {
             my $pos = $raf->Tell();
+            # extract the TIFF preview
+            my $len = Get32u(\$dos, 20);
+            my $val = $exifTool->ExtractBinary($base, $len, 'TIFFPreview');
+            if (defined $val and $val =~ /^(MM\0\x2a|II\x2a\0|Binary)/) {
+                $exifTool->HandleTag($tagTablePtr, 'TIFFPreview', $val);
+            } else {
+                $exifTool->Warn('Bad TIFF preview image');
+            }
             # extract information from TIFF in DOS header
             # (set Parent to '' to avoid setting FileType tag again)
             my %dirInfo = (
@@ -225,7 +321,6 @@ sub ProcessPS($$)
 # parse the postscript
 #
     my ($buff, $mode, $endToken);
-    my $tagTablePtr = GetTagTable('Image::ExifTool::PostScript::Main');
     my (@lines, $altnl);
     if ($/ eq "\x0d") {
         $altnl = "\x0a";
@@ -243,6 +338,11 @@ sub ProcessPS($$)
                 # split into separate lines
                 @lines = split /$altnl/, $data, -1;
                 $data = shift @lines;
+                if (@lines == 1 and $lines[0] eq $/) {
+                    # handle case of DOS newline data inside file using Unix newlines
+                    $data .= $lines[0];
+                    undef @lines;
+                }
             }
         }
         if ($mode) {
@@ -252,6 +352,8 @@ sub ProcessPS($$)
             } elsif ($data !~ /^$endToken/i) {
                 if ($mode eq 'XMP') {
                     $buff .= $data;
+                } elsif ($mode eq 'Document') {
+                    # ignore embedded documents 
                 } else {
                     # data is ASCII-hex encoded
                     $data =~ tr/0-9A-Fa-f//dc;  # remove all but hex characters
@@ -259,7 +361,7 @@ sub ProcessPS($$)
                 }
                 next;
             }
-        } elsif ($data =~ /^(%{1,2})(Begin)(_xml_packet|Photoshop|ICCProfile)/i) {
+        } elsif ($data =~ /^(%{1,2})(Begin)(_xml_packet|Photoshop|ICCProfile|Binary)/i) {
             # the beginning of a data block
             my %modeLookup = (
                 _xml_packet => 'XMP',
@@ -267,9 +369,17 @@ sub ProcessPS($$)
                 photoshop   => 'Photoshop',
             );
             $mode = $modeLookup{lc($3)};
+            unless ($mode or @lines) {
+                # skip binary data
+                $raf->Seek($1, 1) or last if $data =~ /^%{1,2}BeginBinary:\s*(\d+)/i;
+                next;
+            }
             $buff = '';
             $endToken = $1 . ($2 eq 'begin' ? 'end' : 'End') . $3;
             next;
+        } elsif ($data =~ /^(%{1,2})(Begin)(Document)/i) {
+            $mode = 'Document';
+            next;
         } elsif ($data =~ /^<\?xpacket begin=.{7,13}W5M0MpCehiHzreSzNTczkc9d/) {
             # pick up any stray XMP data
             $mode = 'XMP';
@@ -281,33 +391,32 @@ sub ProcessPS($$)
             my ($tag, $val) = ($1, $2);
             # only allow 'ImageData' to have single leading '%'
             next unless $data =~ /^%%/ or $1 eq 'ImageData';
-            # extract information from PostScript tags in comments
-            $val =~ s/\x0d*\x0a*$//;        # remove trailing CR, LF or CR/LF
-            if ($val =~ s/^\((.*)\)$/$1/) { # remove brackets if necessary
-                $val =~ s/\) \(/, /g;       # convert contained brackets too
-            }
+            # decode comment string (reading continuation lines if necessary)
+            $val = DecodeComment($val, $raf, \@lines);
             $exifTool->HandleTag($tagTablePtr, $tag, $val);
             next;
         } else {
             next;
         }
         # extract information from buffered data
-        my %dirInfo = (
-            DataPt => \$buff,
-            DataLen => length $buff,
-            DirStart => 0,
-            DirLen => length $buff,
-            Parent => 'PostScript',
-        );
-        my $subTablePtr = GetTagTable("Image::ExifTool::${mode}::Main");
-        unless ($exifTool->ProcessDirectory(\%dirInfo, $subTablePtr)) {
-            $exifTool->Warn("Error processing $mode information in PostScript file");
+        if ($mode ne 'Document'){
+            my %dirInfo = (
+                DataPt => \$buff,
+                DataLen => length $buff,
+                DirStart => 0,
+            	DirLen => length $buff,
+            	Parent => 'PostScript',
+            );
+            my $subTablePtr = GetTagTable("Image::ExifTool::${mode}::Main");
+            unless ($exifTool->ProcessDirectory(\%dirInfo, $subTablePtr)) {
+                $exifTool->Warn("Error processing $mode information in PostScript file");
+            }
+            undef $buff;
         }
         undef $mode;
-        undef $buff;
     }
     $/ = $oldsep;   # restore original separator
-    $mode and PSErr($exifTool, "unterminated $mode data");
+    $mode and $mode ne 'Document' and PSErr($exifTool, "unterminated $mode data");
     return 1;
 }
 
@@ -344,7 +453,7 @@ Currently doesn't handle continued lines ("%+" syntax).
 
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
